// ==UserScript==
// @name         Twitch 領取小奇點 + 移除 Raid Referrer + 刷新緩衝區
// @namespace    https://gjptag.jsvtb.cc/
// @icon         https://gjptag.jsvtb.cc/widget/favicon_gjptag.ico
// @version      2.3
// @description  移除 Twitch 網址中的 ?referrer=raid，保持分頁活躍，並領取 Twitch 小奇點，並且按下按鈕後重置小奇點獲取值
// @author       GJPtAg
// @match        *://*.twitch.tv/*
// @match        *://twitch.tv/*
// @exclude      *://*.jsvtb.cc/*
// @exclude      *://jsvtb.cc/*
// @run-at       document-end
// @grant        GM_getValue
// @grant        GM_setValue
// @grant        GM_addStyle
// ==/UserScript==

(function () {
    'use strict';

    function removeReferrerRaid() {
        let url = new URL(window.location.href);
        if (url.searchParams.has('referrer') && url.searchParams.get('referrer') === 'raid') {
            url.searchParams.delete('referrer');
            window.history.replaceState(null, '', url.href);
        }
    }

    function setupFakeMute(video) {
        if (video.dataset.gjptagMute === "true") return;

        if (video.volume > 0) return;

        video.dataset.gjptagMute = "true";

        if (!video.audioCtx) {
            video.audioCtx = new (window.AudioContext || window.webkitAudioContext)();
            video.source = video.audioCtx.createMediaElementSource(video);
            video.gainNode = video.audioCtx.createGain();

            video.gainNode.gain.value = 0.01;
            video.source.connect(video.gainNode).connect(video.audioCtx.destination);
        }

        video.volume = 0.01;

        video.addEventListener("volumechange", () => {
            if (video.volume > 0) {
                video.gainNode.gain.value = 1;
                video.dataset.gjptagMute = "false";
            } else if (video.dataset.gjptagMute === "true") {
                video.gainNode.gain.value = 0;
            }
        });
    }

    function findAndMuteVideos() {
        document.querySelectorAll("video:not([data-gjptag-mute])").forEach(video => setupFakeMute(video));
    }

    const observer = new MutationObserver(findAndMuteVideos);
    observer.observe(document.body, { childList: true, subtree: true });

    window.addEventListener("load", function () {
        removeReferrerRaid();
        findAndMuteVideos();
    });

    GM_addStyle(`
        #gift-count__button {
            margin: 0px 0px 0px -10px;
	}
        #latency-update-btn {
            margin: 0px 4px;
	}

        #gift-count__button,
        #latency-update-btn {
            min-width: 29px;
	    text-align:center;
            padding: 5px;
            font-size: 12px;
            height: 30px;
	    white-space: nowrap;
            background-color: #9147ff;
            color: white;
            border: none;
            border-radius: 5px;
            cursor: pointer;
        }

        #gift-count__button:hover,
        #latency-update-btn:hover {
            background-color: #772ce8;
        }

        #gift-count__button:active,
        #latency-update-btn:active {
            background-color: #5c16c5;
        }

        #gift-container {
            display: flex;
            justify-content: space-between;
        }

        #gift-container > div {
            display: flex;
            align-items: center;
        }
    `);

    let Gift = GM_getValue(location.href, 0);
    let GiftCountButton = document.createElement("button");
    GiftCountButton.id = "gift-count__button";

    function formatGiftCount(count) {
        if (count >= 10000) {
            let result = (Math.floor(count / 10000 * 10) / 10).toFixed(1);
            if (result.endsWith(".0")) {
                result = result.slice(0, -2);
            }
            return result + "w";
        } else if (count >= 1000) {
            let result = (Math.floor(count / 1000 * 10) / 10).toFixed(1);
            if (result.endsWith(".0")) {
                result = result.slice(0, -2);
            }
            return result + "k";
        }
        return count.toLocaleString();
    }

    GiftCountButton.textContent = formatGiftCount(Gift);

    GiftCountButton.onclick = function() {
        Gift = 0;
        GM_setValue(location.href, Gift);
        GiftCountButton.textContent = formatGiftCount(Gift);
    };

    GiftCountButton.onmouseenter = function() {
        GiftCountButton.style.zIndex = "999";
        let GiftCounttop = document.createElement("div");
        GiftCounttop.id = "gift-GiftCounttop";
        GiftCounttop.textContent = `自動獲得 ${Gift.toLocaleString()} 次 按下可歸零`;

        GiftCounttop.style.position = "absolute";
        GiftCounttop.style.backgroundColor = "#FFFFFF";
        GiftCounttop.style.color = "#000000";
        GiftCounttop.style.padding = "4px";
        GiftCounttop.style.borderRadius = "4px";
        GiftCounttop.style.fontSize = "12px";
        GiftCounttop.style.whiteSpace = "nowrap";
        GiftCounttop.style.zIndex = "1000";
        GiftCounttop.style.transform = "translateX(-100%)";
        GiftCounttop.style.transform = "translateY(-100%)";

        let rect = GiftCountButton.getBoundingClientRect();
        GiftCounttop.style.left = `${rect.left + rect.width / 2 - GiftCounttop.offsetWidth / 2 - 80}px`;
        GiftCounttop.style.top = `${rect.top - GiftCounttop.offsetHeight - 10}px`;

        document.body.appendChild(GiftCounttop);

        GiftCountButton.GiftCounttop = GiftCounttop;
    };

    GiftCountButton.onmouseleave = function() {
        GiftCountButton.style.zIndex = "";
        if (GiftCountButton.GiftCounttop) {
            document.body.removeChild(GiftCountButton.GiftCounttop);
            delete GiftCountButton.GiftCounttop;
        }
    };

    let latencyButton = document.createElement("button");
    latencyButton.id = "latency-update-btn";
    latencyButton.innerText = "▶▶";

    latencyButton.onclick = function () {
        let video = document.querySelector('video');
        if (video) {
            video.currentTime = video.buffered.end(0) - 0.1;
            video.play();
        }
    };

    latencyButton.onmouseenter = function() {
        latencyButton.style.zIndex = "999";
        let latencytip = document.createElement("div");
        latencytip.id = "gift-latencytip";
        latencytip.textContent = `刷新緩衝`;

        latencytip.style.position = "absolute";
        latencytip.style.backgroundColor = "#FFFFFF";
        latencytip.style.color = "#000000";
        latencytip.style.padding = "4px";
        latencytip.style.borderRadius = "4px";
        latencytip.style.fontSize = "12px";
        latencytip.style.whiteSpace = "nowrap";
        latencytip.style.zIndex = "1000";
        latencytip.style.transform = "translateX(-100%)";
        latencytip.style.transform = "translateY(-100%)";

        let rect = latencyButton.getBoundingClientRect();
        latencytip.style.left = `${rect.left + rect.width / 2 - latencytip.offsetWidth / 2 - 30}px`;
        latencytip.style.top = `${rect.top - latencytip.offsetHeight - 10}px`;

        document.body.appendChild(latencytip);

        latencyButton.latencytip = latencytip;
    };

    latencyButton.onmouseleave = function() {
        latencyButton.style.zIndex = "";
        if (latencyButton.latencytip) {
            document.body.removeChild(latencyButton.latencytip);
            delete latencyButton.latencytip;
        }
    };

    function insertGiftButton() {
        let container = document.querySelector('.chat-input__buttons-container div');
        if (container && !document.getElementById("gift-count__button")) {
            let giftContainer = document.createElement("div");
            giftContainer.id = "gift-container";

            let pointsDiv = document.createElement("div");
            pointsDiv.appendChild(GiftCountButton);

            let latencyDiv = document.createElement("div");
            latencyDiv.appendChild(latencyButton);

            giftContainer.appendChild(pointsDiv);
            giftContainer.appendChild(latencyDiv);

            container.after(giftContainer);
        }
    }

    function collectTwitchPoints() {
        let bonusButton = document.querySelector('.claimable-bonus__icon');
        if (bonusButton) {
            bonusButton.click();

            Gift = GM_getValue(location.href, 0) + 1;
            GM_setValue(location.href, Gift);

            GiftCountButton.textContent = formatGiftCount(Gift);
        }
    }

    function updateChatPlaceholder() {
        let video = document.querySelector("video");

        if (video) {
            let bufferedEnd = video.buffered.length > 0 ? video.buffered.end(0) : 0;
            let latency = (bufferedEnd - video.currentTime).toFixed(2);
            let placeholderDiv = document.querySelector('div.chat-wysiwyg-input__placeholder');

            if (placeholderDiv) {
                placeholderDiv.textContent = `緩衝: ${latency}s`;
            }
        }
    }

    function monitorTwitchErrors() {
        let errorElements = document.querySelectorAll(".content-overlay-gate__allow-pointers");
        if (errorElements.length > 0) {
            let errorText = errorElements[0].textContent.trim();
            if (errorText.includes("#2000") || errorText.includes("#3000") || errorText.includes("#4000")) {
                console.log("偵測到 Twitch 錯誤:", errorText, "自動重新整理...");
                location.reload();
            }
        }
    }

    window.onload = function () {
        setInterval(insertGiftButton, 1000);
        setInterval(collectTwitchPoints, 1000);
        setInterval(updateChatPlaceholder, 1000);
        setInterval(monitorTwitchErrors, 1000);
    };
})();
